{*******************************************************}
{                                                       }
{    Copyright  1995-2002 by Lucian Radulescu          }
{    mailto  :  lucian@ez-delphi.com                    }
{    http    :  http://www.ez-delphi.com                }
{                                                       }
{*******************************************************}

{$I ALLLIB.DEF}

unit EZConsts;

interface

uses
  {$IFDEF WIN32} Windows,
  {$ELSE}        WinTypes, WinProcs,
  {$ENDIF WIN32}
  SysUtils,
  {$IFNDEF ATLEAST_D2} VER, {$ENDIF ATLEAST_D2}
  Classes;

const

  LangStrId    : String = '\\StringFileInfo\\040904E4\\';
  LangStrIdGE  : String = '\\StringFileInfo\\040704E4\\';

  { TDateTime Time Interval Constants }
  MILLISECOND  = (1 / 86400000);
  SECOND       = (1 / 86400);
  MINUTE       = (1 / 1440);
  HOUR         = (1 / 24);
  DAY          = 1;
  WEEK         = 7;
  YEAR         = 365;

  BoolStr      : array [Boolean] of String = ('FALSE', 'TRUE');
  PluralStrs   : array [Boolean] of String = ('', 's');
  YesNoStr     : array [Boolean] of String = ('No', 'Yes');
  CRLF         : String = #13#10;

{$IFNDEF WIN32}
type
  DWORD = Longint;
  UINT  = Word;
{$ENDIF}

var

  OpenLog                 : String =
    '--------------------------------- OPEN LOG -------------------------------------'#13#10;
  CloseLog                : String =
    '--------------------------------- CLOSE LOG ------------------------------------'#13#10;

  AppPath                 : String   = '';
  AppDir                  : String = '';
  AppDrive                : Byte;
  INITIALIZATION_OKAY     : Boolean  = FALSE;

function TransferException: Exception;
function Application_ExeName: string;

function GetResourceInfoStrLang(const AppName, ResName, Lang: String): String;
function GetResourceInfoStr(const AppName, ResName: String): String; overload;
function GetResourceInfoStr(const ResName: String): String; overload;
function InternalReadResource(const ResName: string; M: TMemoryStream): Boolean;
function GetResBuffer(ResName: String): PChar;

function GetTimeAsString(FormatIndex: Integer): String;
function GetTimeAsStringEx(TheTime: TDateTime; FormatIndex: Integer): String;

{$IFDEF ATLEAST_D2}
function GetTotalDuration( Duration: TDateTime ): String;

resourcestring
{$ELSE}
const
  MAX_PATH = 255;
{$ENDIF ATLEAST_D2}
  SDuplicateItem = 'List does not allow duplicates ($0%x)';
  cap_MsgBox_Confirm = 'Please Confirm';
  cap_MsgBox_Error = 'Error';
  cap_MsgBox_Inform = 'Information';
  cap_MsgBox_Warn = 'Warning';

{$IFDEF ATLEAST_D2}
var
{$ELSE}
const
{$ENDIF ATLEAST_D2}

  VER_FILEDESCRIPTION : String = '';
  VER_FILEVERSION     : String = '';
  VER_LEGALCOPYRIGHT  : String = '';
  VER_COMPANYNAME     : String = '';
  VER_LEGALTRADEMARKS : String = '';
  VER_INTERNALNAME    : String = '';
  VER_SPECIALBUILD    : String = '';
  VER_PRIVATEBUILD    : String = '';

var
  IsWin32s             : Boolean = FALSE;
  IsWindowsNT40        : Boolean = FALSE;
  IsWindows95          : Boolean = FALSE;
  IsWindows95OSR2      : Boolean = FALSE;
  IsWindows98          : Boolean = FALSE;
  IsWindows98SE        : Boolean = FALSE;
  IsWindows2000        : Boolean = FALSE;
  IsWindowsMe          : Boolean = FALSE;
  IsWindowsXP          : Boolean = FALSE;

  IsWinNT              : Boolean = FALSE;

implementation

uses
  Consts;

type
  PRaiseFrame = ^TRaiseFrame;
  TRaiseFrame = record
    NextRaise       : PRaiseFrame;
    ExceptAddr      : Pointer;
    ExceptObject    : TObject;
    {$IFDEF ATLEAST_D2}
    ExceptionRecord : PExceptionRecord;
    {$ENDIF ATLEAST_D2}
  end;

function TransferException: Exception;
begin
  if Pointer(RaiseList) = nil then Result := nil
  else begin
    Result := Exception( PRaiseFrame( RaiseList )^.ExceptObject );
    PRaiseFrame( RaiseList )^.ExceptObject := nil;
  end;
end;

function Application_ExeName: string;
var
  FileName: array [0..MAX_PATH] of Char;
begin
  if GetModuleFileName(0, FileName, MAX_PATH) > 0 then
    Result := StrPas(FileName)
  else
    Result := '';
end;

function GetVersionData(FileName, QueryValue: String; Buff: PChar): PChar;
var
  xFileName: array[0..255] of char;
  xQueryValue: array[0..255] of char;
  Buffer: Pointer;
  VerSize: LongInt;                 { Length of buffer for info }
  PData: PChar;                     { Buffer to place the data structure }
  VersionHandle: DWORD;             { Information for use by GetFileVersionInfo }
  Len:                              { Length, in bytes, of the version-information value. }
    {$IFDEF ATLEAST_D2}DWORD;{$ELSE}WORD;{$ENDIF ATLEAST_D2}
begin
  StrCopy(Buff, #0);
  StrPCopy(xFileName, FileName);
  StrPCopy(xQueryValue, QueryValue);

  VerSize := GetFileVersionInfoSize(xFileName, VersionHandle);
  if VerSize > 0 then
  begin
    PData := StrAlloc(VerSize+1);
    try
      if GetFileVersionInfo(xFileName, VersionHandle, VerSize, PData) then
        if VerQueryValue(PData, xQueryValue, Buffer, Len) then
          StrCopy(Buff, Buffer);
    finally
      StrDispose(PData);
    end;
  end;
  Result := Buff;
end;

function GetResourceInfoStrLang( const AppName, ResName, Lang: String ): String;
var
  Buffer: PChar;
begin
  Buffer := StrAlloc(1000);
  FillChar(Buffer^, 1000, #0);
  try
    Result := StrPas(GetVersionData(AppName, Format('%s%s', [Lang, ResName]), Buffer));
  finally
    StrDispose(Buffer);
  end;
end;

function GetResourceInfoStr( const AppName, ResName: String ): String;
var
  Buffer: PChar;
begin
  Buffer := StrAlloc(1000);
  FillChar(Buffer^, 1000, #0);
  try
    Result := StrPas(GetVersionData(AppName, Format('%s%s', [LangStrId, ResName]), Buffer));
  finally
    StrDispose(Buffer);
  end;
end;

function GetResourceInfoStr( const ResName: String ): String;
begin
  Result := GetResourceInfoStr( Application_ExeName, ResName );
end;

function InternalReadResource( const ResName: string; M: TMemoryStream ): Boolean;
var
  HRsrc: THandle;
  HInst: THandle;
  R    :
    {$IFDEF ATLEAST_D3}   TResourceStream;
    {$ELSE}               TStream;
  Handle: Integer;
    {$ENDIF ATLEAST_D3}

begin
  HInst :=
    {$IFDEF ATLEAST_D3}   FindResourceHInstance(HInstance);
    {$ELSE}               HInstance;
    {$ENDIF ATLEAST_D3}

  HRsrc := FindResource( HInst,
    {$IFDEF ATLEAST_D2}  PChar(ResName),
    {$ELSE}              @ResName[1],
    {$ENDIF ATLEAST_D2}
                        RT_RCDATA );

  Result := HRsrc <> 0;
  if not Result then Exit;

{$IFNDEF ATLEAST_D2}
  Handle := AccessResource(HInstance, HRsrc);
  if Handle = 0 then
    raise EResNotFound.Create(FmtLoadStr(SResNotFound, [ResName]));
{$ENDIF ATLEAST_D2}
  R :=
    {$IFDEF ATLEAST_D2}   TResourceStream.Create( HInst, ResName, RT_RCDATA );
    {$ELSE}               THandleStream.Create(Handle);
    {$ENDIF ATLEAST_D2}


  try
    M.CopyFrom( R, R.Size );
    M.Position := 0;
  finally
    R.Free;
  end;
end;

function GetTimeAsStringEx(TheTime: TDateTime; FormatIndex: Integer): String;
begin
  if TheTime < 0 then Result := ''
  else
  case FormatIndex of
    0: Result := FormatDateTime('yyyy/mm/dd HH:nn:ss.zzz', TheTime);
    1: Result := FormatDateTime('yyyy/mm/dd hh:nn:ss', TheTime);
    2: Result := FormatDateTime(ShortDateFormat+' hh:mm:ss', TheTime);
    3: Result := FormatDateTime('nn:ss', TheTime);
    4: Result := FormatDateTime('yyyy/mm/dd hh:nn', TheTime);
    5: Result := FormatDateTime('HH:MM:SS.ZZZ', TheTime);

    94: if TheTime = 0 then Result := 'created'
        else Result := FormatDateTime('yyyy/mm/dd hh:nn', TheTime);
  end;
end;

function GetTimeAsString(FormatIndex: Integer): String;
begin
  Result := GetTimeAsStringEx(Now, FormatIndex);
end;

(*
function GetTotalDuration(Duration: TDateTime): String;
var
  T, S: TTimeStamp;
  Hour, Min, Sec, Mil: Longint;
begin
  Hour := 0;
  Min  := 0;
  Sec  := 0;
  Result := '';

  T := DateTimeToTimeStamp(0);
  S := DateTimeToTimeStamp(Duration);
  S.Date := S.Date - T.Date;
  Mil := S.Time;
  if Mil > 0 then
  begin
    Sec := Mil div 1000;
    if Sec > 0 then
    begin
      Mil := Mil - Sec * 1000;
      Min := Sec div 60;
      if Min > 0 then
      begin
        Sec := Sec - Min * 60;
        Hour:= Min div 60;
        if Hour > 0 then
        begin
          Min := Min - Hour * 60;
          if Hour > 23 then
          begin
            Inc(S.Date, Hour div 24);
            Hour := Hour - (Hour div 24) * 24;
          end;
        end;
      end;
    end;
  end;

  if S.Date > 0 then
    if S.Date = 1 then Result := CT_1_DAY
    else Result := Format (CT_DAYS, [S.Date]);

  if Hour > 0 then
    if Hour = 1 then
      Result := Format (CT_1_HOUR, [Result]);
    else
      Result := Format (CT_HOURS, [Result, Hour]);

  if Min > 0 then
    if Min = 1 then
      Result := Format (CT_1_MIN, [Result]);
    else
      Result := Format (CT_MIN, [Result, Min]);

  if (Sec > 0) or (Mil > 0); then
    if Sec = 1 then
      Result := Format (CT_1_SEC, [Result, Mil]);
    else
      Result := Format (CT_SEC, [Result, Sec, Mil]);

  Result := Trim(Result);

  if Result = '' then Result := CT_0_SEC;
end;
*)

{$IFDEF ATLEAST_D2}
function GetTotalDuration(Duration: TDateTime): String;
var
  T, S: TTimeStamp;
  Hour, Min, Sec, Mil: Longint;
begin
  Hour := 0;
  Min  := 0;
  Sec  := 0;
  Result := '';

  T := DateTimeToTimeStamp(0);
  S := DateTimeToTimeStamp(Duration);
  S.Date := S.Date - T.Date;
  Mil := S.Time;
  if Mil > 0 then
  begin
    Sec := Mil div 1000;
    if Sec > 0 then
    begin
      Min := Sec div 60;
      if Min > 0 then
      begin
        Sec := Sec - Min * 60;
        Hour:= Min div 60;
        if Hour > 0 then
          Min := Min - Hour * 60;
      end;
    end;
  end;
  Result := Format ('%.2d:%.2d:%.2d', [S.Date*24 + Hour, Min, Sec]);
end;
{$ENDIF ATLEAST_D2}

function GetResBuffer(ResName: String): PChar;
var
  ResIdHandle, ResDataHandle: THandle;
  ResDataPtr: Pointer;
  Size: Cardinal;
begin
  Result := nil;
  ResIDHandle := FindResource( HInstance,
    {$IFDEF ATLEAST_D2}        PChar(ResName),
    {$ELSE}                    @ResName[1],
    {$ENDIF ATLEAST_D2}
                               RT_RCDATA );
  if ResIDHandle = 0 then Exit;
  ResDataHandle := LoadResource(HInstance, ResIDHandle);
  if ResDataHandle = 0 then Exit;
  ResDataPtr := Pointer(ResDataHandle);
  Size := StrLen(ResDataPtr);
  if (ResDataPtr <> nil) then
  begin
    Result := StrAlloc(Size+1);
    StrLCopy(Result, PChar(ResDataPtr), Size);
  end;
end;

initialization

  if Win32MajorVersion >= 5 then
    if Win32MinorVersion >= 1 then IsWindowsXP := TRUE
    else IsWindows2000 := TRUE
  else
    if Win32MajorVersion > 3 then
      if Win32Platform = VER_PLATFORM_WIN32_NT then IsWindowsNT40 := TRUE
      else
        //Win32BuildNumber := Win32BuildNumber and $FFFF;
        if Win32MinorVersion >= 90 then IsWindowsMe := TRUE
        else
          if Win32MinorVersion >= 10 then
            if Win32BuildNumber >= 2222 then IsWindows98SE := TRUE
            else IsWindows98 := TRUE
          else
            if Win32BuildNumber >= 1000 then IsWindows95OSR2 := TRUE
            else IsWindows95 := TRUE
    else IsWin32s := TRUE;

  IsWinNT := (Win32Platform and VER_PLATFORM_WIN32_NT) <> 0;

  VER_FILEDESCRIPTION := GetResourceInfoStr('FILEDESCRIPTION');
  VER_FILEVERSION     := GetResourceInfoStr('FILEVERSION');
  VER_LEGALCOPYRIGHT  := GetResourceInfoStr('LEGALCOPYRIGHT');
  VER_COMPANYNAME     := GetResourceInfoStr('COMPANYNAME');
  VER_LEGALTRADEMARKS := GetResourceInfoStr('LEGALTRADEMARKS');
  VER_INTERNALNAME    := GetResourceInfoStr('INTERNALNAME');
  VER_SPECIALBUILD    := GetResourceInfoStr('SPECIALBUILD');
  VER_PRIVATEBUILD    := GetResourceInfoStr('PRIVATEBUILD');

  if AppPath = '' then
  begin
    AppPath  := ExtractFilePath( ParamStr(0) );
    AppDir   := ExtractFileDir ( ParamStr(0) );
    AppDrive := Ord( AppDir[1] ) - (ord('A') - 1);
  end;
  ChDir( AppDir );
end.
